require 'httparty'
require 'json'
require 'rspec'

module ECUtils

  ScheduleDay = Struct.new(:id, :advisor_id, :day, :start_time, :end_time)

  class ECAdvisorApis
    include HTTParty
    include RSpec::Matchers

    attr_accessor :everclear_session

    base_uri "https://#{ECUtils.sub_domain.downcase}.everclearbeta.com"

    def authorized_headers
      {
        'Cookie': "everclear_session=#{everclear_session}",
        'Content-Type': 'application/json',
        'Accept': 'application/json'
      }
    end

    def sign_in_as_advisor(email, password='12345678')
      options = {
        body: {
          'advisor[email]': email,
          'advisor[password]': password,
          'advisor[remember_me]': 0,
          'commit': 'Log in'
        }
      }
      response = self.class.post('/advisors/sign_in', options)
      @everclear_session = response.headers['Set-Cookie'].scan(/everclear_session[^;]*/)[0].split(/[=;]/)[1]
    end

    def send_text_message(message, seeker_id)
      options = {
        body: {
          'cleaner': 'raw',
          'content': message,
          'recipient_id': seeker_id
        }.to_json,
        headers: authorized_headers
      }
      response = self.class.post('/text_chat/messages.json', options)
      puts response.body
      expect(response.code).to eq 200
    end

    def set_availability(data)
      options = {
        body: {
          'availability': data
        }.to_json,
        headers: authorized_headers
      }
      response = self.class.put('/profile/availability', options)
      puts response.body
      expect(response.code).to eq 200
    end

    def send_photo_message(receiver_id, photo_data)
      options = {
        body: {
          'receiver_id': receiver_id,
          'recipient_id': receiver_id,
          'ref_id': Time.now.to_i,
          'photo_message[]': photo_data
        },
        headers: {
          'Cookie': "everclear_session=#{everclear_session}",
          'Accept': '*/*'
        }
      }
      puts options
      response = self.class.post('/text_chat/photo_messages', options)
      puts response.body
      expect(response.code).to eq 200
    end

    def set_phone_rate(rate)
      options = {
        body: {
          'availability': {
            "price_per_minute": rate
          }
        }.to_json,
        headers: authorized_headers
      }
      response = self.class.patch('/profile/availability.json', options)
      puts response.body
      expect(response.code).to eq 200
    end

    def update_email(advisor_id, advisor_email, password)
      options = {
        body: {
          'email': advisor_email,
          'id': advisor_id,
          'password': password
        }.to_json,
        headers: authorized_headers
      }

      response = self.class.patch("/advisors_basic_info/#{advisor_id}", options)
      puts response.body
    end

    def update_password(advisor_id, advisor_email, password, new_password)
      options = {
        body: {
          'email': advisor_email,
          'id': advisor_id,
          'password': password,
          'new_password': new_password,
          'new_password_confirmation': new_password
        }.to_json,
        headers: authorized_headers
      }
      puts options
      response = self.class.patch("/advisors_basic_info/#{advisor_id}", options)
      puts response.body
    end

    def get_my_clients(direction, field, name, page=1)
      query = {}
      unless direction.nil?
        query['direction'] = direction
      end
      unless field.nil?
        query['field'] = field
      end
      unless name.nil?
        query['name'] = name
      end
      unless page.nil?
        query['page'] = page
      end
      options = {
          query: query,
          headers: authorized_headers
      }

      response = self.class.get('/clients.json', options)
      puts response.body
      JSON.parse(response.body)['clients']
    end

    def get_advisor_feedbacks
      options = {
        headers: authorized_headers
      }
      response = self.class.get('/feedbacks.json?page=1', options)
      body_response = JSON.parse(response.body)
      body_response['feedbacks'] unless body_response.empty?
    end

    def get_advisor_pinned_feedbacks
      options = {
        headers: authorized_headers
      }
      response = self.class.get('/feedbacks/pinned.json', options)
      JSON.parse(response.body)
    end

    def get_chat_contacts
      options = {
        headers: authorized_headers
      }
      response = self.class.get('/chat_contacts.json', options)
      JSON.parse(response.body)
    end

    def reset_unread_message(seeker_id)
      options = {
          headers: authorized_headers
      }
      response = self.class.put("/text_chat/message_counts/#{seeker_id}/reset_unread_messages", options)
      JSON.parse(response.body)
    end

    def unpin_feedback(feedback_id)
      options = {
        headers: authorized_headers
      }
      self.class.post("/feedbacks/#{feedback_id}/unpin.json", options)
    end

    def unblock_client(user_id)
      options = {
        headers: authorized_headers
      }
      self.class.post("/clients/#{user_id}/unblock.json", options)
    end

    def get_lapsed_client(min_earn=5)
      options = {
        query: { minimum_earn: min_earn },
        headers: authorized_headers
      }
      response = self.class.get('/advisor/lapsed_clients', options)
      JSON.parse(response.body)['clients']
    end

    def get_availability
      options = {
        headers: authorized_headers
      }
      response = self.class.get('/profile/availability', options)
      puts response.body
      expect(response.code).to eq 200
      response['availability']
    end

    def update_schedule(schedules=[])
      options = {
          body: { typical_schedule_blocks: schedules }.to_json,
          headers: authorized_headers
      }
      response = self.class.post('/advisor/typical_schedule.json', options)
      JSON.parse(response.body)['typical_schedule_blocks']
    end

    def update_timezone(timezone)
      options = {
          body: { timezone: timezone }.to_json,
          headers: authorized_headers
      }
      response = self.class.patch('/advisors_basic_info/update_timezone', options)
      JSON.parse(response.body)['data']
    end

    def update_away_message(message)
      options = {
          body: { away_message: message }.to_json,
          headers: authorized_headers
      }
      response = self.class.patch('/advisor_profile/update_away_message', options)
      puts response.body
      expect(response.code).to eq 200
    end

    def update_advisor_profile(name, biography, summary)
      options = {
          body: {
            name: name,
            biography: biography,
            summary: summary,
            tools: { numerology: true, tarot: true },
            topics: {love_and_relationships: true, career: true}
          }.to_json,
          headers: authorized_headers
      }
      response = self.class.patch('/advisor_profile', options)
      expect(response.code).to eq 200
    end

    def get_timezone_list
      options = {
          headers: authorized_headers
      }
      response = self.class.get('/advisors_basic_info/timezone_list', options)
      expect(response.code).to eq 200
      response['available_timezones']
    end

    def accept_chat_session(chat_id)
      options = {
          body: {type: "advisor_accept", success: true}.to_json,
          headers: authorized_headers
      }
      response = self.class.patch("/chats/#{chat_id}.json", options)
    end

    def decline_chat_session(chat_id)
      options = {
          body: {type: "advisor_accept_failed", success: false}.to_json,
          headers: authorized_headers
      }
      response = self.class.patch("/chats/#{chat_id}.json", options)
    end

    def start_chat_conference(chat_id)
      options = {
          body: {type: "start_conference", success: true}.to_json,
          headers: authorized_headers
      }
      response = self.class.patch("/chats/#{chat_id}.json", options)
    end

    def send_ppm_chat(chat_id, content)
      options = {
          body: {content: content}.to_json,
          headers: authorized_headers
      }
      response = self.class.post("/chats/#{chat_id}/messages", options)
    end

    def end_chat_session(chat_id)
      options = {
          body: {type: "end_session", success: true}.to_json,
          headers: authorized_headers
      }
      puts response = self.class.patch("/chats/#{chat_id}", options)
    end

    def send_typing_signal(chat_id)
      options = {
          body: {}.to_json,
          headers: authorized_headers
      }
      response = self.class.post("/chats/#{chat_id}/typing", options)
    end

    def start_promoted_profile
      options = {
          headers: authorized_headers
      }
      response = self.class.post('/promoted_profile_campaign.json', options)
      expect(response.code).to eq 200
    end

    def get_promoted_profile_campaign
      options = {
          headers: authorized_headers
      }
      response = self.class.get('/promoted_profile_campaign.json', options)
    end
  end
end